import ui
from tones import beep
import pickle
import globalPluginHandler
import appModuleHandler
import gui, wx
from winKernel import kernel32
import winUser
import os.path
import addonHandler
import api
hw_path=os.path.join(os.path.abspath(os.path.dirname(__file__)), 'hiddenwindows.dat')
PROCESS_ALL_ACCESS=2035711
priority_list=[_("background"), _("below normal"), _("normal"), _("above normal"), _("high"), _("realtime")]
priorityclasses=[64, 16384, 32, 32768, 128, 256]
addonHandler.initTranslation()
def error_beep():
	beep(300, 180)
def get_handle(pid=None):
	if pid == None:
		pid = kernel32.GetCurrentProcessId()
	return kernel32.OpenProcess(PROCESS_ALL_ACCESS, True, pid)
def setPriority(pid=None, priority=2):
	handle=get_handle(pid)
	return kernel32.SetPriorityClass(handle, priorityclasses[priority])!=0
def getPriority(pid=None):
	handle = get_handle(pid)
	return priorityclasses.index(kernel32.GetPriorityClass(handle))
def get_focus_pid():
	return api.getFocusObject().processID
def save_hidden_windows(windows):
	f=open(hw_path, 'wb')
	pickle.dump(windows, f)
	f.close
def load_hidden_windows():
	if not os.path.isfile(hw_path):
		return {}
	f=open(hw_path, 'rb')
	ws=pickle.load(f)
	f.close()
	return ws
def hide_window():
	current=api.getForegroundObject()
	appName=appModuleHandler.getAppNameFromProcessID(current.processID, True)
	title=current.name
	wrong_titles={None: _('no title'), 'Program Manager': _('this is a program manager'), 'Taskbar': _('this is a taskbar')}
	if title in wrong_titles:
		error_beep()
		ui.message(wrong_titles[title])
		return
	current=current.windowHandle
	winUser.user32.ShowWindowAsync(current, 0)
	beep(80, 180)
	return [current, title, appName]
def show_window(handle):
	beep(180, 180)
	winUser.user32.ShowWindowAsync(handle, 3)
	winUser.user32.SetForegroundWindow(handle)
class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	def __init__(self, *args, **kwargs):
		globalPluginHandler.GlobalPlugin.__init__(self, *args, **kwargs)
		self.hidden=load_hidden_windows()
		self.stack=1
		self.lastHidden=None
		self.wl_dialog=None
	def hs_window(self, num):
		w=self.hidden.get(num)
		if w and w[0]!=api.getForegroundObject().windowHandle and winUser.isWindow(w[0])!=0:
			show_window(w[0])
			if num==self.lastHidden:
				self.lastHidden=None
			del self.hidden[num]
		else:
			w=hide_window()
			if not w:
				return
			self.hidden[num]=w
		save_hidden_windows(self.hidden)
		return True
	def clear_destroyed_windows(self):
		changed=False
		for k in list(self.hidden.keys()):
			if not winUser.isWindow(self.hidden[k][0]):
				changed=True
				del self.hidden[k]
		if changed:
			save_hidden_windows(self.hidden)
	def window_list(self):
		self.clear_destroyed_windows()
		if not self.hidden:
			ui.message(_("There are no windows currently hidden."))
			return
		wlist=[]
		for i, w in self.hidden.items():
			wlist.append(([int(x) for x in i.split('-')], _("{n} {title}, from process {process}").format(title=w[1], process=w[2], n=i)))
		wlist.sort(key=lambda x: x[0])
		wlist=[w[1] for w in wlist]
		if not self.wl_dialog:
			self.wl_dialog=Window_list_dialog(gui.mainFrame, self, wlist)
		else:
			self.wl_dialog.update(wlist)
		if not self.wl_dialog.IsShown():
			gui.mainFrame.prePopup()
			self.wl_dialog.Show()
			self.wl_dialog.Centre()
			gui.mainFrame.postPopup()
	def script_hideFirstAvailable(self, gesture):
		self.stack=1
		selected=1
		while self.hidden.get(f'{self.stack}-{selected}'):
			selected+=1
			if selected>10:
				self.stack+=1
				selected=1
		lastHidden=f'{self.stack}-{selected}'
		if self.hs_window(lastHidden):
			self.lastHidden=lastHidden
			wx.CallLater(1, ui.message, lastHidden)
	def script_unhideLastWindow(self, gesture):
		if not self.lastHidden:
			ui.message(_("There is no window recorded as last hidden window"))
			return
		self.hs_window(self.lastHidden)
	def script_windowlist(self, gesture):
		self.window_list()
	def script_killProcess(self, gesture):
		handle = kernel32.OpenProcess(1, 0, get_focus_pid())
		beep(90, 300) if kernel32.TerminateProcess(handle, 0) else error_beep()
	def script_set_priority(self, gesture):
		num=int(gesture.mainKeyName[-1])-1
		setPriority(get_focus_pid(), num)
		self.say_priority(num)
	def say_priority(self, n):
		ui.message(_("Priority: {}.").format(priority_list[n]))
	def script_get_priority(self, gesture):
		self.say_priority(getPriority(get_focus_pid()))
	def script_hs_window(self, gesture):
		num=int(gesture.mainKeyName[-1])
		num=10 if num==0 else num
		self.hs_window(f'{self.stack}-{num}')
	def script_changeTitle(self, gesture):
		window=api.getForegroundObject()
		d = wx.TextEntryDialog(gui.mainFrame, _("Enter the new title for this window:"), _("Change window title"))
		def callback(result):
			if result == wx.ID_CANCEL:
				return
			winUser.user32.SetWindowTextA(window.windowHandle, d.GetValue().encode('1251'))
		gui.runScriptModalDialog(d, callback)
#		gui.messageBox(_("Can't change window title"), _("Error"), wx.OK|wx.ICON_ERROR)
	def script_previous_stack(self, gesture):
		if self.stack>1:
			self.stack-=1
		ui.message(_("stack {0}").format(self.stack))
	def script_next_stack(self, gesture):
		if self.stack<404:
			self.stack+=1
		ui.message(_("stack {0}").format(self.stack))
	def script_cycleWindows(self, gesture):
		nwp=0
		nwn=0
		try:
			fg=api.getForegroundObject().parent
			fgp=fg.appModule
		except Exception:
			ui.message("Can't retrieve window information for this object.")
		try:
			nwn=fg.simpleNext if fg.simpleNext.appModule==fgp else 0
			nwp=fg.simplePrevious if fg.simplePrevious.appModule==fgp else 0
		except AttributeError:
			ui.message("This is not a valid window. You are probably on the desktop or the start button.")
			return
		if not (nwn or nwp):
			ui.message("This window has no top level windows to cycle to.")
			return
		if nwn and nwp:
			ui.message("Multiple top level windows detected.")
			return
		if nwn:
			nwn.setFocus()
		else:
			nwp.setFocus()
	__gestures={
"kb:shift+windows+leftArrow":"previous_stack",
"kb:shift+windows+rightArrow":"next_stack",
"kb:windows+shift+tab":"cycleWindows",
"kb:shift+windows+l": "windowlist",
"kb:windows+shift+h": "hideFirstAvailable",
"kb:control+shift+windows+h": "unhideLastWindow",
"kb:windows+F4": "killProcess",
"kb:nvda+windows+tab": "get_priority",
"kb:windows+shift+t": "changeTitle",
}
	for n in range(10):
		__gestures[f"kb:shift+windows+{n}"]="hs_window"
	for n in range(1, 7):
		__gestures[f"kb:nvda+windows+{n}"]="set_priority"

class Window_list_dialog(wx.Dialog):
	def __init__(self, parent, pl, wlist, title=_("Window List")):
		wx.Dialog.__init__(self, parent, title=title)
		self.plugin=pl
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		tasksSizer = wx.BoxSizer(wx.VERTICAL)
		tasksLabel = wx.StaticText(self, -1, label=_("Window list"))
		tasksSizer.Add(tasksLabel)
		self.listBox = wx.ListBox(self, style=wx.LB_SINGLE, size=(550, 250))
		tasksSizer.Add(self.listBox, proportion=8)
		self.update(wlist)
		mainSizer.Add(tasksSizer)
		buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
		showButton=wx.Button(self, wx.ID_OK, _("&Show Window"))
		buttonsSizer.Add(showButton)
		cancelButton = wx.Button(self, wx.ID_CANCEL)
		self.Bind(wx.EVT_BUTTON, self.onShow, showButton)
		buttonsSizer.Add(cancelButton)
		self.SetSizer(mainSizer)
		mainSizer.Fit(self)
	def onShow(self, evt):
		num=self.listBox.GetString(self.listBox.GetSelection()).split()[0]
		self.plugin.hs_window(num)
		self.Close()
	def update(self, wlist):
		self.listBox.SetItems(wlist)
		self.listBox.SetSelection(0)
		self.listBox.SetFocus()
